use glium::Display;
use imgui::Ui;
use imgui::color::ImColor32;

use crate::ui_generic::drag_area::DragArea;
use crate::storage::Storage;

const KF_CHAN_HEIGHT: f32 = 20.0;
const KF_CHAN_TITLE_DEFAULT_COLOR: ImColor32 = 
    ImColor32::from_rgb(38, 38, 38);
const KF_CHAN_TITLE_HOVER_COLOR: ImColor32 =
    ImColor32::from_rgb(79, 79, 79);
const KF_CHAN_CONTENT_DEFAULT_COLOR: ImColor32 =
    ImColor32::from_rgb(16, 16, 16);
const KF_CHAN_CONTENT_HOVER_COLOR: ImColor32 =
    ImColor32::from_rgb(54, 54, 54);

pub struct KeyframeEditor {
    pub drag_area: DragArea
}

impl KeyframeEditor {
    pub fn new(size: [f32; 2]) -> KeyframeEditor {
        KeyframeEditor {
            drag_area: DragArea::new(size, [0.0, 0.0])
        }
    }

    pub fn build(
        &mut self,
        ui: &Ui,
        display: &Display,
        storage: &mut Storage
    ) {
        self.drag_area.build(
            &ui,
            &display.gl_window().window(),
            |draw_list, area_min, area_max, offset| {
                let mouse_y = ui.io().mouse_pos[1];

                let origin: [f32; 2];

                if storage.keyframe_channels.len() == 0 {
                    let text_pos: [f32; 2] = [
                        area_min[0] + (area_max[0] - area_min[0]) / 2.0 - 50.0,
                        area_min[1] + (area_max[1] - area_min[1]) / 2.0 - 2.5
                    ];

                    draw_list.add_text(
                        text_pos,
                        KF_CHAN_TITLE_HOVER_COLOR,
                        "No keyframes to show."
                    );

                    return;
                }

                let origin = [
                    area_min[0] + offset[0],
                    area_min[1] + offset[1]
                ];

                let mut chan_y: f32 = 1.0;
                let mut next_chan_y: f32 = chan_y + KF_CHAN_HEIGHT;

                for (idx, chan) in storage.keyframe_channels.iter().enumerate() {
                    let title_bg: ImColor32;
                    let content_bg: ImColor32;

                    if (
                        (mouse_y < origin[1] + next_chan_y) &&
                        (mouse_y > origin[1] + chan_y)
                    ) {
                        content_bg = KF_CHAN_CONTENT_HOVER_COLOR;
                        title_bg = KF_CHAN_TITLE_HOVER_COLOR;
                    } else {
                        content_bg = KF_CHAN_CONTENT_DEFAULT_COLOR;
                        title_bg = KF_CHAN_TITLE_DEFAULT_COLOR;
                    }

                    let chan_title_start: [f32; 2] = [
                        origin[0] + 1.0,
                        origin[1] + chan_y + 1.0
                    ];

                    let chan_title_end: [f32; 2] = [
                        origin[0] + 100.0,
                        origin[1] + next_chan_y + 1.0
                    ];

                    draw_list
                        .add_rect(
                            chan_title_start,
                            chan_title_end,
                            title_bg
                        )
                        .filled(true)
                        .thickness(0.0)
                        .build();

                    let title_pos: [f32; 2] = [
                        origin[0] + 1.0 + 3.0,
                        origin[1] + chan_y + 2.0
                    ];

                    draw_list.add_text(
                        title_pos,
                        ImColor32::WHITE,
                        String::from(&chan.name)
                    );

                    let chan_content_end: [f32; 2] = [
                        area_max[0],
                        chan_title_start[1]
                    ];

                    draw_list
                        .add_rect(
                            chan_title_end,
                            chan_content_end,
                            content_bg
                        )
                        .filled(true)
                        .thickness(0.0)
                        .build();
                    
                    if true {
                        let border_start = [
                            origin[0],
                            origin[1] + next_chan_y
                        ];

                        let border_end = [
                            area_max[0],
                            origin[1] + next_chan_y
                        ];

                        draw_list
                            .add_line(
                                border_start,
                                border_end,
                                title_bg
                            )
                            .build();
                    }

                    chan_y += KF_CHAN_HEIGHT;
                    next_chan_y += KF_CHAN_HEIGHT;
                }
            }
        );
    }
}
